import 'dart:io';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:video_player/video_player.dart';
import 'material/chewie_route_page_builder.dart';

class ChatVideoPlayer extends StatefulWidget {
  final VideoPlayerController videoPlayerController;
  final String videoPath, videoUrl;
  final bool looping, autoPlay, showAllControls;
  final double aspectRatio;
  final Function callbackFunction;

  const ChatVideoPlayer({
    this.videoPlayerController,
    this.videoPath = "",
    this.videoUrl = "",
    this.looping = true,
    this.autoPlay = true,
    this.showAllControls = true,
    this.aspectRatio = 16 / 9,
    this.callbackFunction,
  });

  @override
  _ChatVideoPlayerState createState() => _ChatVideoPlayerState();
}

class _ChatVideoPlayerState extends State<ChatVideoPlayer> {
  ChewieController _chewieController;
  VideoPlayerController _videoPlayerController;
  String _filePath, _fileUrl;
  bool _looping, _autoPlay;
  double _aspectRatio;

  @override
  void initState() {
    super.initState();
    _initData();
    _initVideoPlayerController();
    _chewieController = ChewieController(
        videoPlayerController: _videoPlayerController,
        looping: _looping,
        aspectRatio: _aspectRatio,
        autoPlay: _autoPlay,
        autoInitialize: true,
        errorBuilder: (context, errorMessage) {
          return Center(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Text(
                errorMessage,
                textAlign: TextAlign.center,
                style: TextStyle(color: Colors.white),
              ),
            ),
          );
        });
  }

  void _initData() {
    _filePath = widget.videoPath;
    _fileUrl = widget.videoUrl;
    _looping = widget.looping;
    _autoPlay = widget.autoPlay;
    _aspectRatio = widget.aspectRatio;
    if (widget.callbackFunction != null) {
      widget.callbackFunction();
    }
  }

  void _initVideoPlayerController() {
    if (widget.videoPlayerController == null) {
      if (_filePath != "" && _filePath.isNotEmpty) {
        _videoPlayerController = VideoPlayerController.file(File(_filePath));
      } else {
        _videoPlayerController =
            VideoPlayerController.network(Uri.parse(_fileUrl).toString());
      }
    } else {
      _videoPlayerController = widget.videoPlayerController;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: MediaQuery.of(context).size.height / 3,
      color: Colors.black,
      child: Chewie(
        controller: _chewieController,
        colorData: _getColorData(),
        showAllControls: widget.showAllControls,
      ),
    );
  }

  MaterialControlsColorData _getColorData() {
    return  MaterialControlsColorData(
      isChild: true,
      bottomBarColor: Colors.blue,
      iconBackgroundColor: Colors.white,
      iconColor: Colors.blue,
      timerTextColor: Colors.white,
      playedColor: Colors.yellowAccent,
    );
  }

  @override
  void dispose() {
    _chewieController.dispose();
    _videoPlayerController.dispose();
    super.dispose();
  }
}