import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'chat_textmessage_widget.dart';
import 'chat_voicemessage_widget.dart';
import 'material/image_viewer.dart';
import 'chat_video_player.dart';
const kTextMessage = "text";
const kImageMessage = "image";
const kVideoMessage = "video";
const kVoiceMessage = "voice";
const kFileMessage = "file";
class ChatBubble extends StatelessWidget {
  final Widget senderImage;
  final String text, time, type;
  final bool isMe;

  ChatBubble({
    @required this.senderImage,
    this.text = "",
    this.time = "",
    this.type = "",
    this.isMe = false,
  });

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Row(
        mainAxisAlignment:
        isMe ? MainAxisAlignment.start : MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          SenderImageWidget(
            isMe,
            senderImageWidget: senderImage,
          ),
          Flexible(
            child: Column(
              crossAxisAlignment:
              isMe ? CrossAxisAlignment.start : CrossAxisAlignment.end,
              children: [
                _mapToChatWidget(context),
                Text(
                  time,
                  maxLines: 1,
                  softWrap: true,
                  overflow: TextOverflow.ellipsis,
                  style: TextStyle(
                    color: Colors.black.withOpacity(0.5),
                    fontSize: 12,
                  ),
                ),
              ],
            ),
          ),
          SenderImageWidget(
            !isMe,
            senderImageWidget: senderImage,
          ),
        ],
      ),
    );
  }

  Widget _mapToChatWidget(BuildContext context) {
    switch (type) {
      case kImageMessage:
        return _getImageMessage(context);
        break;
      case kVideoMessage:
        return ChatVideoPlayer(
          autoPlay: false,
          looping: false,
          videoUrl: text,
        );
        break;
      case kVoiceMessage:
        return ChatVoiceMessageWidget(url: text);
        break;

      default:
        return ChatTextMessageWidget(
          text: text,
          isMe: isMe,
        );
    }
  }

  Widget _getImageMessage(BuildContext context) {
    return GestureDetector(
      onTap: () => Navigator.push(
        context,
        MaterialPageRoute(
          builder: (context) => ImageViewer(text),
        ),
      ),
      child: AspectRatio(
        aspectRatio: 16 / 9,
        child: ClipRRect(
          borderRadius: BorderRadius.circular(12.0),
          child: Image.network(
            text,
            fit: BoxFit.fitWidth,
          ),
        ),
      ),
    );
  }
}

class SenderImageWidget extends StatelessWidget {
  final bool toggleWidgetLocation;
  final Widget senderImageWidget;

  const SenderImageWidget(
      this.toggleWidgetLocation, {
        Key key,
        this.senderImageWidget,
      }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (toggleWidgetLocation) {
      return Container(
        padding: const EdgeInsets.symmetric(horizontal: 15),
        child: senderImageWidget,
      );
    } else {
      return Container();
    }
  }
}
